package alien;

import joptsimple.OptionParser;
import joptsimple.OptionSet;
import joptsimple.OptionSpec;
import protoj.lang.Command;
import protoj.lang.CommandStore;
import protoj.lang.StandardProject;
import alien.core.AlienCore;

/**
 * A typical class that demonstrates some of the features available from the
 * ProtoJ library. Without much effort the Alien project acquires use-cases for
 * compilation, junit testing, assembly and tar creation, uploading to google
 * code and more.
 * <p>
 * To set up simply take a copy of all the jars from the ProtoJ lib directory
 * and place them in the lib directory for this Alien project. Then you can go
 * ahead and try executing a selection of the following commands from the alien
 * project bin directory:
 * 
 * <pre>
 * bin$ ./alien.sh retrieve
 * bin$ ./alien.sh compile
 * bin$ ./alien.sh help
 * bin$ ./alien.sh &quot;help find-alien&quot;
 * bin$ ./alien.sh archive
 * bin$ ./alien.sh test
 * bin$ ./alien.sh javadoc
 * bin$ ./alien.sh &quot;find-alien -gx milky way&quot;
 * bin$ ./alien.sh &quot;upload-gc bobuser secretpassword&quot;
 * bin$ ./alien.sh clean
 * </pre>
 * 
 * Notice that the ivy and compile commands are executed first. That's because
 * ProtoJ is part of the Alien project rather than an outside component and
 * therefore library download and compilation need to happen before any other
 * project use-case can be invoked.
 * 
 * 
 * @author Ashley Williams
 * 
 */
public final class AlienProject {
	/**
	 * Each command that gets executed in The AlienCore class causes this
	 * AlienProject virtual machine to be started with the same command. However
	 * this time when that same command is executed, its body will be invoked.
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		new AlienProject(args).dispatchCommands();
	}

	/**
	 * The top level protoj pojo, obtained from the {@link AlienCore} class.
	 */
	private StandardProject project;

	/**
	 * Enables a number of standard project use-cases as well as adding a brand
	 * new Alien project specific use-case command.
	 * 
	 * @param args
	 */
	public AlienProject(String[] args) {
		// contains the protoj StandardProject instance with all the standard
		// use-cases built in
		AlienCore core = new AlienCore(args);

		project = core.getProject();

		// enables artifacts to be uploaded to a maven repository using scp
		project.initPublish("scp://myserver:/mydir");
		project.getPublishFeature().initProvider("wagon-ssh", "1.0-beta-2");

		// enables junit testing with the specified amount of memory
		project.initJunit("16m");

		// enables classes jar file creation with the specified jar name
		project.getArchiveFeature().addClasses("alien");

		// enables source file archive creation with the specified jar name
		project.getArchiveFeature().addSources("alien");

		// enables javadoc archive creation with the specified jar name and
		// maximum memory that the javadoc tool should allocate
		project.getArchiveFeature().addJavadoc("alien", "16m");

		// enables jar publication to maven using the specified pom.xml file
		// with any generated sources and javadoc archives and signature files
		String gpgOptions = "--armor --local-user bob --output %2$s --detach-sign %1$s";
		project.getArchiveFeature().getClassesArchive().initPublish("alien",
				"/alien/pom.xml", gpgOptions);

		// enables the project artifacts to be uploaded to google code
		project.initUploadGoogleCode("alien");

		// adds a command for a new project use-case which is configured to run
		// in a new vm with 16m of memory and then call AlienProject.findAlien()
		new FindAlienCommand(project);

		// of course ProtoJ will never include every permutation so...

		// possibility: a release command that calls compile, test then tar and
		// copies to a remote machine using scp, all using the ProtoJ api
		// new AlienReleaseCommand(project);

		// possibility: a webservice command that creates java classes from wsdl
		// using the apache axis ant task
		// new WebserviceCommand(project);
	}

	/**
	 * Dispatches command line arguments to the commands that arrived in the
	 * main method of this class.
	 */
	public void dispatchCommands() {
		project.getDispatchFeature().dispatchCommands();
	}

	/**
	 * Probably doesn't rival SETI as yet, just displays a message to stdout.
	 * 
	 * @param galaxy
	 */
	public void findAlien(String galaxy) {
		StringBuilder builder = new StringBuilder();
		builder.append("No life found in the ");
		builder.append(galaxy);
		builder.append(" galaxy, please try again later!");
		System.out.println(builder.toString());
	}

	/**
	 * Responsible for adding a new {@link Command} to the {@link CommandStore}.
	 * The third-party JOpt Simple api is also used to configure the command
	 * line options - in this case a mandatory 'gx' option that represents the
	 * galaxy name.
	 */
	public final class FindAlienCommand {
		private final Command commandImpl;
		private final OptionSpec<String> gx;

		public FindAlienCommand(StandardProject project) {
			// adds our new find-alien command to the project command store
			commandImpl = project.getCommandStore().addCommand("find-alien",
					"16m", new Runnable() {
						public void run() {
							// extracts the -gx option that was set up below
							OptionSet options = commandImpl.getOptions();
							findAlien(options.valueOf(gx));
						}
					});
			commandImpl
					.initHelpString("search the -gx galaxy for extra terristrial life");

			// allows another name for the command
			commandImpl.initAliases("find-ET");

			// tells the body (above) to expect a -gx command line option
			OptionParser parser = commandImpl.getParser();
			gx = parser.accepts("gx").withRequiredArg();
		}

	}
}
